import pandas as pd
from .utils.connect import connection

class SQLExecutor :
    """SQL执行器类，提供统一的数据库操作接口"""

    def __init__( self , sql_config ,database=None) :
        self.mydb , self.mycursor = connection( sql_config,database )

    # 关闭数据库连接
    def close( self ) :
        self.mycursor.close()
        self.mydb.close()

    # 提交并关闭数据库连接
    def commit_close( self ) :
        self.mydb.commit()
        self.close()

    # sql 语句执行器
    def execute( self , sql , params = None , commit = False , self_close = False ) :
        """
        SQL语句执行方法，自动判断是否批量执行
        :param params: 参数，单个参数为元组，批量参数为元组列表，如 [(value1, value2), (value3, value4)]
        """
        try :
            if params :
                # 判断是否为批量参数（列表且第一个元素是元组）
                if isinstance(params, list) and len(params) > 0 and isinstance(params[0], (tuple, list)):
                    self.mycursor.executemany(sql, params)
                else:
                    # 如果是列表但不是批量参数，转换为元组
                    if isinstance(params, list):
                        params = tuple(params)
                    self.mycursor.execute(sql, params)
            else :
                self.mycursor.execute(sql)

            if commit :
                # 提交事务
                self.mydb.commit()

        except Exception as e :
            print(f"sql: {sql} \n params:{params}")
            # 如果发生错误，回滚事务
            if commit :
                self.mydb.rollback()
            self.mydb.close()
            raise Exception(f"SQL execute failed: {str(e)}")

        if self_close :
            self.close()


    # 定义解析结果程序(格式化返回结果)
    def fetch_format( self , sql , fetch_mode , output_format = "" , show_count = False , data_label = None ,
                      params = None , self_close = False ) :
        """
        定义解析结果程序(格式化返回结果)
        :param sql: SQL语句
        :param fetch_mode: 获取模式
        :param output_format: 输出格式
        :param show_count: 是否显示结果数量
        :param data_label: 数据标签
        :param params: 参数
        :param self_close: 是否自动关闭连接
        :return: 查询结果
        """
        from .tools.result_formatter import fetch_format as fetch_format_func
        return fetch_format_func(self, sql, fetch_mode, output_format, show_count, data_label, params, self_close)


    # 插入数据
    def insert( self , table_name , insert_fields , skip_duplicate = False, commit = False , self_close = False ) :
        from .utils.insert import insert as insert_func
        insert_func(self, table_name, insert_fields, skip_duplicate, commit, self_close)


    # 更新数据
    def update( self , table_name , update_fields , where_conditions , commit = False , self_close = False ) :
        """
        通用的SQL更新执行器方法，支持动态构造WHERE子句

        :param table_name: 表名
        :param update_fields: 需要更新的字段和值，格式为字典，如 {'field1': 'value1', 'field2': 'value2'}
        :param where_conditions: WHERE条件，格式为字典，如 {'field1': 'value1', 'field2': 'value2'}
        :param commit: 是否自动提交
        :param self_close: 是否自动关闭连接
        :return: None
        """
        from .utils.update import update as update_func
        update_func(self, table_name, update_fields, where_conditions, commit, self_close)

    # 删除数据
    def delete( self , table_name , where_conditions , commit = False , self_close = False ) :
        """
        通用的SQL删除执行器方法，支持动态构造WHERE子句

        :param table_name: 表名
        :param where_conditions: WHERE条件，格式为字典，如 {'field1': 'value1', 'field2': 'value2'}
        :param commit: 是否自动提交
        :param self_close: 是否自动关闭连接
        :return: None
        """
        from .utils.delete import delete as delete_func
        delete_func(self, table_name, where_conditions, commit, self_close)


    # 选择数据
    def select( self , table_names , select_fields , where_conditions = None, order_by = None , limit = None ,
                join_conditions = None ,
                self_close = False , fetch_config = None ) :
        """
        通用的SQL查询执行器方法，支持JOIN操作
        :param table_names: 表名，可以是字符串或列表
        :param select_fields: 要查询的字段列表
        :param where_conditions: WHERE条件，格式为字典
        :param order_by: ORDER BY子句
        :param limit: LIMIT子句
        :param join_conditions: JOIN条件，格式为字典，如 {"join_type": "JOIN", "conditions": ["field1", "=", "field2"]}
        :param self_close: 是否自动关闭连接
        :param fetch_config: 获取配置
        :return: 查询结果
        """
        from .utils.select import select as select_func
        return select_func(self, table_names, select_fields, where_conditions, order_by, limit, join_conditions, self_close, fetch_config)


    def fetch_and_response( self,table_names , select_fields , where_conditions = None, 
        join_conditions=None,fetch_config = None,format_func=None , self_close = True ) :
        """
        通用的产品数据获取与格式化方法
        """
        
        if fetch_config is None :
            fetch_config = { "fetch_mode" : "all" , "output_format" : "df_dict" , "data_label" : None }
        order_by = fetch_config.get("order_by", None)
        limit = fetch_config.get("limit", None)
        try :
            # 使用默认的select方法
            result = self.select( table_names , select_fields , where_conditions ,order_by, limit,
                                            join_conditions, self_close , fetch_config )
            success = True
            message = "success"
            
            if format_func is not None :
                try :
                    result = format_func(result)
                except Exception as e :
                    success = False
                    message = f"sql run success > format failed > {result} > {format_func.__name__} > {str(e)}"

        except Exception as e :
            success = False
            result = { }
            error_msg = str(e)
            message = f"sql run failed > {error_msg}"
            if "No result set to fetch from" in error_msg:
                message += f''' : note > The query statement did not return a result set. 
                 Please check if the database connection self.mycursor in the pip package was closed prematurely.'''
            success = False
            result = { }
            
        return { "success" : success , "result" : result , "message" : message }

    # 支持复杂查询的执行方法
    def execute_query(self, sql, params=None, fetch_config=None, self_close=False):
        """
        执行自定义SQL查询
        :param sql: SQL语句
        :param params: 参数
        :param fetch_config: 获取配置
        :param self_close: 是否自动关闭连接
        :return: 查询结果
        """
        if fetch_config is None:
            fetch_config = {}
        
        fetch_mode = fetch_config.get("fetch_mode", "all")
        output_format = fetch_config.get("output_format", "df_dict")
        show_count = fetch_config.get("show_count", False)
        data_label = fetch_config.get("data_label", [])
        
        result = self.fetch_format(sql, fetch_mode, output_format, show_count, data_label, params, self_close)
        return result


    # 将 table 中的字段和字段类型，导出为md格式文件
    def export_table_md( self , table_name , save_path , self_close = True ) :
        """
        将 table 中的字段和字段类型，导出为md格式文件
        :param table_name: 表名
        :param self_close: 是否自动关闭连接
        :return: None
        """
        from .tools.table_export import export_table_md as export_table_md_func
        export_table_md_func(self, table_name, save_path, self_close)



from ..executor import SQLExecutor

def insert(executor: SQLExecutor, table_name, insert_fields, skip_duplicate=False, commit=False, self_close=False):
    """
    通用的SQL插入执行器方法，
    :param executor: SQLExecutor 实例
    :param table_name: 表名
    :param insert_fields: 字段和值，格式为字典，如 {'field1': 'value1', 'field2': 'value2'}
    :param skip_duplicate: 是否跳过重复数据--基于主键或唯一索引(包含唯一复合索引)判断
        注意: 只有主键或当索引被明确设置为UNIQUE时才会触发跳过重复记录的行为,普通索引(如INDEX)不会导致跳过重复记录。
    :param commit: 是否自动提交
    :param self_close: 是否自动关闭连接
    :return: None
    """
    if isinstance(insert_fields, dict):
        # 单条插入
        sql = f'''INSERT INTO {table_name} 
                    ({', '.join(insert_fields.keys())}) VALUES ({', '.join(['%s'] * len(insert_fields))});'''
        values = tuple(insert_fields.values())
        executor.execute(sql, values, commit, self_close)
    elif isinstance(insert_fields, list):
        # 批量插入

        # 获取字段名（假设所有字典的键相同）
        field_names = ', '.join(insert_fields[0].keys())

        # 构造占位符，例如：(%s, %s)
        placeholders = '(' + ', '.join(['%s'] * len(insert_fields[0])) + ')'

        # 构造SQL语句
        insert_keyword = 'INSERT IGNORE' if skip_duplicate else 'INSERT'
        sql = f'''{insert_keyword} INTO {table_name} ({field_names}) VALUES {placeholders}'''

        # 构造参数列表
        values = [tuple(item.values()) for item in insert_fields]

        # 执行SQL
        executor.execute(sql, values, commit, self_close)
    else:
        executor.close()
        raise ValueError("insert_fields must be a dict or a list of dicts")


import mysql.connector

# 获取数据库连接和游标
def connection( sql_config ,database=None ) :

    """
    建立数据库连接并返回连接对象和游标对象

    Args:
        sql_config (object): 数据库配置对象，包含host, port, user, passwd, default_database等属性
        database (str, optional): 数据库名称，database参数优先使用，默认使用 sql_config.default_database
    Returns:
        tuple: (数据库连接对象, 游标对象)
    """
    if database is not None :
        database = database
    elif not hasattr(sql_config, 'default_database') or sql_config.default_database is None:
        database = "new_schema"
    else:
        database = sql_config.default_database
        
    try:
        # 新版本使用password参数和推荐参数
        # buffered=True - 缓冲查询结果，避免多次查询时出现'Unread result found'错误
        # pool_size=5 - 连接池最大连接数，控制并发连接数量，
        # pool_reset_session=True - 连接返回池时重置会话变量，确保连接状态干净
        # pool_name="shop_pool" - 连接池名称，用于标识和管理连接池 
        # 如果设置了 pool_reset_session 就必须设置 pool_name ，省略会报错- AttributeError: 
        # Pool name 'rm-wz93y5aqe2f5gvu5uto.mysql.rds.aliyuncs.com_3306_root_yqq_new_schema' is too long
        # use_pure=True - 使用纯Python实现而非C扩展，提高兼容性，减少外部依赖（实测，设置在为False会导致连接失败！）

        mydb = mysql.connector.connect(
            host=sql_config.host,
            port=sql_config.port,
            user=sql_config.user,
            password=sql_config.passwd,
            database=database,
            buffered=True,
            use_pure=True
         )
        mycursor = mydb.cursor(buffered=True)
    except TypeError:
        # 旧版本兼容模式
        mydb = mysql.connector.connect(
            host=sql_config.host,
            port=sql_config.port,
            user=sql_config.user,
            passwd=sql_config.passwd,
            database=database,
            use_pure=True
        )
        mycursor = mydb.cursor()
    
    # 检查版本是否过时
    if tuple(map(int, mysql.connector.__version__.split('.')[:2])) < (9, 4):
        print(f"警告: MySQL连接器版本{mysql.connector.__version__}已过时，建议升级到9.4.0或更高版本")
        print("pip install --upgrade mysql-connector-python")
    return mydb , mycursor